#!/usr/bin/python

import numpy as np
import matplotlib.pyplot as mpl
import sys
import os
import os.path

OUTDIR = os.path.join(sys.path[0], 'fig3-data')

def eprint(*args, **kwargs):
    if 'file' in kwargs.keys():
        kwargs.pop('file')
    print(*args, file=sys.stderr, **kwargs)


def erotate(matrix, deg):
    theta = np.radians(deg)
    c, s = np.cos(theta), np.sin(theta)
    R = np.array(((c, -s), (s, c)))

    return np.dot(matrix, R.T)


def dataout(f, x, y):
    outfile = os.path.join(OUTDIR, f)
    mat = np.column_stack((x, y))
    np.savetxt(outfile, mat, delimiter='\t')


def main():

    if os.path.exists(OUTDIR) and (not os.path.isdir(OUTDIR)):
        eprint("%s exists but is not a directory; exiting" % OUTDIR)
        sys.exit(1)

    if not os.path.exists(OUTDIR):
        eprint("Creating %s" % OUTDIR)
        os.mkdir(OUTDIR)

    x = np.arange(1, 501)

    # switching data
    yb = np.random.normal(150*np.log(x), 40) + 5000
    dyb = np.diff(yb)
    ybrot = erotate(np.column_stack((yb[:-1], dyb)), -45)

    # steady state data
    ys = np.random.normal(-15, 15, x.shape) + np.max(yb) - 60
    dys = np.diff(ys)
    ysrot = erotate(np.column_stack((ys[:-1], dys)), -45)

    # Uncomment these lines to plot the data
    # mpl.scatter(x, ys)
    # mpl.scatter(x, yb)
    # mpl.show()

    dataout('sim-read-orig.dat', x, ys)
    dataout('sim-bias-orig.dat', x, yb)
    eprint("Raw data generated")

    # Uncomment these lines to plot the data
    # mpl.scatter(ys[:-1], dys)
    # mpl.scatter(yb[:-1], dyb)
    # mpl.show()

    dataout('sim-read-diff.dat', ys[:-1], dys)
    dataout('sim-bias-diff.dat', yb[:-1], dyb)
    eprint("Diff data calculated")

    # Uncomment these lines to plot the data
    # mpl.scatter(ysrot.T[0], ysrot.T[1])
    # mpl.scatter(ybrot.T[0], ybrot.T[1])
    # mpl.show()

    dataout('sim-read-rot.dat', ysrot.T[0], ysrot.T[1])
    dataout('sim-bias-rot.dat', ybrot.T[0], ybrot.T[1])
    eprint("Rotated data calculated")

    eprint("Finished")

if __name__ == "__main__":
    main()
